/* ============================================================================
 * File Name: DDS_1.c
 * Version 0.0
 *
 * Description:
 *   DDS24: 24-bit DDS frequency generator component (v 0.0)
 *   Produces two syncronous fractional frequency outputs (out1 and out2)
 *   Outputs out1 and out2 separated by variable 8-bit phase shift [0..255]. 
 *
 * Credits:
 *   based on original IQ_DDS code by <PSoC73>:
 *   http://www.cypress.com/?app=forum&id=2492&rID=88149
 * 
 *   Special thanks to <pavloven>, <vdvorak>, <kabron>
 *
 * Note:
 *   frequency resolution = clock_frequency / 2^24
 *   frequency maximum    = clock_frequency / 2
 *   frequency minimum    = clock_frequency / 2^24
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/

#include <DDS_1.h>

  

#define DDS_RESOLUTION (double) 16777216.0 // 16777216 = 2^24
#define TUNE_WORD_MAX  (uint32)  8388607u  //  8388607 = 2^23 - 1


double Tdiv;		    // time period per bit increment
uint32 tune_word;       // DDS tune word
double SetFreq;         // DDS set frequency 
uint8  PhaseShift;      // out2 phase shift 


//==============================================================================
// Initialize component
// Set input clock frequency and default values for output frequency and phase 
//==============================================================================

void DDS_1_Init()
{   
    // make record of the input clock frequency into period   
    Tdiv = (double) (DDS_RESOLUTION / DDS_1_CLOCK_FREQ);	
    
    DDS_1_SetFrequency( DDS_1_PresetFreq );
    DDS_1_SetPhase( DDS_1_PresetPhase );   
}

//==============================================================================
// Start component
// Initialize and enable DDS component
//==============================================================================

void DDS_1_Start()
{   
    DDS_1_Init();
    DDS_1_Enable();               
}

//==============================================================================
// Enable component
// Have effect only in Software_only and Software_and_Hardware modes 
//==============================================================================

void DDS_1_Enable()
{   
    #if DDS_1_CR_ENABLE
        CY_SET_REG8( DDS_1_DDSCore_sCTRLReg_ctrlreg__CONTROL_REG, 1 ); //software enable
    #endif                    
}

//==============================================================================
// Stop component
// Have effect only in Software_only and Software_and_Hardware modes 
//==============================================================================

void DDS_1_Stop()
{   
    #if DDS_1_CR_ENABLE
        CY_SET_REG8( DDS_1_DDSCore_sCTRLReg_ctrlreg__CONTROL_REG, 0 ); //DDS software stop
    #endif                    
}

//==============================================================================
// Set output frequency
//==============================================================================

uint8 DDS_1_SetFrequency( double freq )
{ 
    uint8 result = 0; // success = 1, fail = 0 
    
    
    #if DDS_1_CONTROL_FREQ_API
        
        uint32 tmp = (uint32) (freq * Tdiv + 0.5);           // calculate tune word
        if ( (tmp < 1) || (tmp > TUNE_WORD_MAX) )  return 0; // fail -> exit if outside of the valid raange 
          
        tune_word = tmp;
        
        CY_SET_REG8( DDS_1_DDSCore_sControl_ControlPhaseA__CONTROL_REG,  tune_word     &0xFF ); //
        CY_SET_REG8( DDS_1_DDSCore_sControl_ControlPhaseB__CONTROL_REG, (tune_word>> 8)&0xFF ); //
        CY_SET_REG8( DDS_1_DDSCore_sControl_ControlPhaseC__CONTROL_REG, (tune_word>>16)&0xFF ); // 
        
        SetFreq = freq; // backup value
        result = 1;     // success
        
    #endif
    
    return result;
}

//==============================================================================
// Set out1-out2 phase shift
// (any integer value will be truncated to uint8)
//==============================================================================

void DDS_1_SetPhase( uint8 Phase ) //todo: rename SetPhase?
{ 	   
    #if (DDS_1_OUT2_ENABLED && DDS_1_CONTROL_PHASE_API)
        CY_SET_REG8( DDS_1_DDSCore_sControl_ControlPhaseD__CONTROL_REG,  Phase & 0xFF ); // write phase shift
        PhaseShift = Phase; //backup 
    #endif    
}

//==============================================================================
//     Helper functions
//==============================================================================

double DDS_1_GetOutFreq() //return actual output frequency 
{	  
    double OutputFreq  = (((double) tune_word) / DDS_RESOLUTION ) * DDS_1_CLOCK_FREQ;   
    return OutputFreq;
}

double DDS_1_GetFrequency() // return current set frequency 
{	  
    return SetFreq;
}

uint8 DDS_1_GetPhase() //return current phase shift
{	
    return PhaseShift;
}

//==============================================================================
/* [] END OF FILE */
